# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 Yiannis Tsagatakis (talos) jtsagata@jmail.com
# 
# gttkTree is written by: Paul Pogonyshev <pogonyshev@gmx.net>
# We include it here for easy installation
# 
# Graphviz interface is written by: 
# Tomás Vírseda, Jose Fonseca
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import logging
import pygraphviz
import gettext
import indicator
import cairo
import pango
import gobject

import helpers as helpers
import AboutThistleTalesDialog
import PreferencesThistleTalesDialog
import preferences

from gtktree_fix  import TreeViewUtilsFixed
from xdot         import DotWidget, XDotParser
from voidsModel   import VoidsModel
from voidsModel   import Conversion

from gettext import gettext as _
gettext.textdomain('thistle-tales')


# This class is meant to be subclassed by ThistleTalesWindow.  It provides
# common functions and some boilerplate.
class BaseThistleTalesWindow(gtk.Window):
    __gtype_name__ = "BaseThistleTalesWindow"
    TYPE_ICONS=['weather-storm','weather-few-clouds-night','weather-few-clouds','preferences-desktop-screensaver']

    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your initialization code in finish_initializing
    
    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated BaseThistleTalesWindow object.
        """
        builder = helpers.get_builder('ThistleTalesWindow')
        new_object = builder.get_object("thistle_tales_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initializing should be called after parsing the UI definition
        and creating a ThistleTalesWindow object with it in order to finish
        initializing the start of the new ThistleTalesWindow instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)
        self.preferences_dialog = None
        self.sun_state = True
        self.show_voids = True
        self.graph_mode = "TB"

        # Optional Launchpad integration
        # This shouldn't crash if not found as it is simply used for bug reporting.
        # See https://wiki.ubuntu.com/UbuntuDevelopment/Internationalisation/Coding
        # for more information about Launchpad integration.
        try:
            import LaunchpadIntegration
            LaunchpadIntegration.add_items(self.ui.helpMenu, 1, False, True)
            LaunchpadIntegration.set_sourcepackagename('thistle-tales')
        except:
            pass

        # Optional application indicator support
        # Run 'quickly add indicator' to get started.
        # More information:
        #  http://owaislone.org/quickly-add-indicator/
        #  https://wiki.ubuntu.com/DesktopExperienceTeam/ApplicationIndicators
        try:
            #from thistle_tales import indicator
            # self is passed so methods of this class can be called from indicator.py
            # Comment this next line out to disable appindicator
            self.indicator = indicator.new_application_indicator(self)
        except:
            pass
        
        self.model = VoidsModel()
        self.model.connect('row-changed',self.on_model_change)
        self.setup_voids_view()
        self.setup_voids_small_view()
        self.setup_graph()
        self.status_time=self.ui.statusbar1.get_context_id("time")
        self.status_msg_id=self.ui.statusbar1.push(self.status_time,"Getting time")
        self.init_update_status_time()
   
    def init_update_status_time(self):
        date = helpers.get_atitd_time().split(",")
        when = date[0]+" "+date[1]+" "+date[2].split()[0]
        self.ui.statusbar1.remove_message(self.status_time, self.status_msg_id)
        self.status_msg_id=self.ui.statusbar1.push(self.status_time,when)
        gobject.timeout_add(1200000,self.init_update_status_time)
                                                   
    def on_model_change(self,treemodel, path, iter):
        self.update_graph()
    
    def setup_graph(self):
        self.graph=DotWidget()
        self.graph.set_size_request(-1,200)
        self.ui.alignment_graph.add(self.graph)
        self.graph.show()
        self.update_graph()
        
    def get_dot_object(self,rankdir="LR"):
        # "TB", "LR", "BT", "RL"
        """ get dot code for converions """
        iter = self.model.get_iter_first()
        G=pygraphviz.AGraph(directed=True,rankdir=rankdir)
        order=1
        while(iter):
            gets  = self.model.get_value(iter,self.model.columns('gets'))
            gives = self.model.get_value(iter,self.model.columns('gives'))
            cond  = self.model.get_value(iter,self.model.columns('when_txt'))
            klass  = self.model.get_value(iter,self.model.columns('klass'))
            active  = self.model.get_value(iter,self.model.columns('active'))
            outval  = self.model.get_value(iter,self.model.columns('outval'))
            G.add_node("Nit", color="red",fillcolor="turquoise", style="filled",shape="box")
            G.add_node("Pot", color="blue",fillcolor="turquoise", style="filled",shape="box")
            
            show= False
            if klass == Conversion.WHEN_ALWAYS:
                show = True
            else:
                if self.is_day() and  klass == Conversion.WHEN_DAY:
                    show = True
                if not self.is_day() and  klass == Conversion.WHEN_NIGHT:
                    show = False
                    
            if gives and active and show:
                color="blue"
                if cond.startswith("Oxy"):
                    color = "Sienna"
                if cond.startswith("Sun"):
                    color = "green"
                style="dashed"
                penwidth=1
                if float(outval) == 10.0:
                    style="solid"
                if float(outval) > 10:
                    style="bold"
                    penwidth=4
                label ="+%s %s[%s]" %(outval, cond, str(order))
                order = order+1
                G.add_edge(gets,gives,label=label,color=color,style=style,penwidth=str(penwidth))
            iter = self.model.iter_next(iter)        
        return G
        
    def update_graph(self):
        """ Update gui graph """
        # TODO swich graph layout
        G = self.get_dot_object(self.graph_mode)
        dotcode=G.string()
        self.graph.set_dotcode(dotcode)
        self.graph.zoom_to_fit()

    def get_void_shoot(self,fname="test.png"):
        """ Get an anotated png file """
        WIDTH = 600
        HEIGHT = 300
        MIN_ZOOM = 0.75
        G = self.get_dot_object(self.graph_mode)
        dotcode = helpers.parse_dotcode(G.string())
        parser = XDotParser(dotcode)
        graph = parser.parse()
        
        zoom_ratio = min(
            float(WIDTH)/float(graph.width),
            float(HEIGHT)/float(graph.height)
        )
        zoom_ratio=max(MIN_ZOOM,zoom_ratio)
        cr_width  = int(zoom_ratio * graph.width+20)
        cr_height = int(zoom_ratio * graph.height+100)
                
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, cr_width, cr_height)
        context = cairo.Context(surface)
        cr = gtk.gdk.CairoContext(context)
        cr.set_source_rgba(1.0, 1.0, 1.0, 1.0)
        cr.paint()
        
        cr.save()
        cr.translate(10,60)
        cr.scale(zoom_ratio,zoom_ratio)
        graph.draw(cr)
        cr.restore()
        
        cr.save()
        cr.translate(10,10)
        layout=cr.create_layout()
        if self.is_day():
            txt = "Daylight Voids"
        else:
            txt = "Moonlight Voids"
        layout.set_text(txt)
        layout.set_font_description( pango.FontDescription("Sans Bold 12") )
        cr.move_to(0,0)
        cr.set_source_rgb(0.0, 0.0, 1.0)
        cr.set_line_width(0.5)
        cr.update_layout(layout)
        cr.layout_path(layout)
        cr.fill()
        
        date = helpers.get_atitd_time().split(",")
        when = date[0]+" "+date[1]
        cr.move_to(0,16)
        cr.set_source_rgb(0.0, 0.0, 0.0)
        layout.set_font_description( pango.FontDescription("Sans Bold 10") )
        layout.set_text(when)
        cr.update_layout(layout)
        cr.layout_path(layout)
        cr.fill()
        
        cr.set_source_rgb(0.4, 0.4, 0.4)
        # TODO Text size?
        cr.move_to(cr_width-250, cr_height-30)
        layout.set_font_description( pango.FontDescription("Sans Bold 8") )
        layout.set_text("Made using Talos ThistleTales.")
        cr.update_layout(layout)
        cr.layout_path(layout)
        cr.fill()
        cr.restore()
        
        surface.write_to_png(fname)
        return (cr,surface)
        
        
        
    def setup_voids_view(self):
        """ Connect main voids view to model attributes """
        
        self.filter = self.model.filter_new()
        #self.filter.set_visible_column(self.model.columns('visibility'))
        self.filter.set_visible_func(self.get_visibility)
        
        #self.model.connect('row-changed',self.on_model_row_change)
        self.ui.voids_view.props.model = self.filter         
        # Code
        self.ui.col_code.add_attribute(self.ui.col_code_txt_rdr, 'text', self.model.columns('name'))
        self.ui.col_code.add_attribute(self.ui.col_code_txt_rdr, 'sensitive', self.model.columns('active'))
        self.ui.col_code.add_attribute(self.ui.col_code_toggle_rdr, 'active', self.model.columns('active'))
        TreeViewUtilsFixed.enable_renderer_editing_filterd (self.ui.col_code_toggle_rdr, self.filter, 'active')
        # Description
        self.ui.col_desc.add_attribute(self.ui.col_desc_rdr, 'text', self.model.columns('description'))
        self.ui.col_desc.add_attribute(self.ui.col_desc_rdr, 'sensitive', self.model.columns('active'))
        
        # Conditions
        self.ui.col_when.add_attribute(self.ui.col_when_rdr, 'text', self.model.columns('when_txt'))
        self.ui.col_when.add_attribute(self.ui.col_when_rdr, 'sensitive', self.model.columns('active'))
        self.ui.col_when.set_cell_data_func(self.ui.col_when_img_rdr, self.renderer_image_klass)
        # Notes
        self.ui.col_notes.add_attribute(self.ui.col_notes_rdr, 'text', self.model.columns('notes'))
        TreeViewUtilsFixed.enable_renderer_editing_filterd (self.ui.col_notes_rdr, self.filter, 'notes')
        # col_when_img
        self.ui.col_klass.add_attribute(self.ui.col_klass_rdr, 'text', self.model.columns('outval'))
        self.ui.col_klass.props.visible=False
        
    def setup_voids_small_view(self):
        filter = self.model.filter_new()
        self.ui.small_voids_view.props.model = filter
        self.ui.small_voids_col.add_attribute(self.ui.small_voids_col_txt, 'text', self.model.columns('name'))
        filter.set_visible_func(self.get_visibility_void)
        self.ui.small_voids_col.set_cell_data_func(self.ui.small_voids_col_img, self.renderer_image_klass)
        filter.refilter()
    
    def get_visibility_void(self,model, iter, user_data=None):
        return not model.get_value(iter,model.columns('active'))
    
    def get_visibility(self,model, iter, user_data=None):
        """ Controls row visibility """
        if self.show_voids == False and model.get_value(iter,model.columns('active')) == False:
            return False
        klass = model.get_value(iter,model.columns('klass'))
        if klass == Conversion.WHEN_HIDDEN:
            return False
        if klass == Conversion.WHEN_ALWAYS:
            return True
        if self.is_day() and  klass == Conversion.WHEN_DAY:
                return True
        if not self.is_day() and  klass == Conversion.WHEN_NIGHT:
            return True
        # Else
        return False
    
    
    def renderer_image_klass(self,column,cell,model,iter):
        """ Choose icon to show if a recipie is day or night """
        types = ['thistle-none','thistle-night','thistle-day']
        cell.set_property('icon-name',BaseThistleTalesWindow.TYPE_ICONS[ model.get_value(iter,self.model.columns('klass')) ])

    def on_mnu_contents_activate(self, widget, data=None):
        helpers.show_uri(self, "ghelp:%s" % helpers.get_help_uri())

    def on_mnu_about_activate(self, widget, data=None):
        """Display the about box for thistle-tales."""
        about = AboutThistleTalesDialog.AboutThistleTalesDialog()
        response = about.run()
        about.destroy()

    def on_btn_filter_toggled(self, widget, data=None):
        self.show_voids = widget.get_active()
        self.filter.refilter()

    def on_graph_orientation_toggled(self,widget, data=None):
        if widget.get_active():
            self.graph_mode = "LR"
        else:
            self.graph_mode = "TB"
        self.update_graph()

    def on_mnu_preferences_activate(self, widget, data=None):
        """Display the preferences window for thistle-tales."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Preferences from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        if self.preferences_dialog is not None:
            logging.debug('show existing preferences_dialog')
            self.preferences_dialog.present()
        else:
            logging.debug('create new preferences_dialog')
            self.preferences_dialog = PreferencesThistleTalesDialog.PreferencesThistleTalesDialog()
            self.preferences_dialog.connect('destroy', self.on_preferences_dialog_destroyed)
            self.preferences_dialog.show()
        # destroy command moved into dialog to allow for a help button

    def on_mnu_quit_activate(self, widget, data=None):
        """Signal handler for closing the ThistleTalesWindow."""
        self.destroy()

    

    def on_destroy(self, widget, data=None):
        """Called when the ThistleTalesWindow is closed."""
        # Clean up code for saving application state should be added here.
        gtk.main_quit()

    def on_preferences_dialog_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the preferences dialog'''
        logging.debug('on_preferences_dialog_destroyed')
        # to determine whether to create or present preferences_dialog
        self.preferences_dialog = None

    def on_btn_zoom_out_clicked(self,widget):
        self.graph.on_zoom_out(None)
    def on_btn_zoom_in_clicked(self,widget):
        self.graph.on_zoom_in(None)
    def on_btn_zoom_fit_clicked(self,widget):
        self.graph.on_zoom_fit(None)
    def on_btn_zoom_100_clicked(self,widget):
        self.graph.on_zoom_100(action)
    
    def on_graph_save_clicked(self,widget):
        #TODO File dialog replace etc ...
        chooser= gtk.FileChooserDialog(
                "Save conversions Image..", self,
                gtk.FILE_CHOOSER_ACTION_SAVE,
                                     (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                      gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        filter = gtk.FileFilter()
        chooser.props.filter = filter
        chooser.props.do_overwrite_confirmation = True
        filter.set_name("Image png")
        filter.add_pattern("*.png")
        filter.add_mime_type("image/png")
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            self.get_void_shoot(chooser.get_filename())
        chooser.destroy()
     
    def on_toggleaction_day_toggled(self,widget,data=None):
        """ Toggle sun off if not active """
        self.filter.refilter()
        if widget.get_active():
            widget.set_property('icon_name','weather-few-clouds')
        else:
            widget.set_property('icon_name','weather-few-clouds-night')
        self.update_graph()
        
    def is_day(self):
        return self.ui.toggleaction_day.get_active()
            

if __name__ == "__main__":
    window = BaseThistleTalesWindow()
    window.show()
    gtk.main()
